<?php
/*
 *	PHP Contest Scoreboard - run contests on your web site
 *
 *	Copyright (C) 2003-2007 Scott Vivian
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU General Public License for more details.
 */

include_once 'config.php';


function entry_form( $entry=null )
{
	global $scoreboard;
	$num_questions = count( $scoreboard['questions'] );

	$HTML = '<p>Email address is used for validation purposes only. When you enter the contest in the future you must use the same name/email
combination in order to accumulate your score on the leader board correctly.</p>' . "\n";

	$HTML .= '<form action="' . $_SERVER[PHP_SELF] . '" method="post" id="contest">' . "\n";
	$HTML .= '<fieldset>' . "\n";
	$HTML .= '<legend>Your details</legend>' . "\n";
	$HTML .= '<div class="required">' . "\n";
	$HTML .= '	<label for="contest-name">Your name</label>' . "\n";
	$HTML .= '	<input type="text" id="contest-name" name="contest[name]" value="' .
		( isset( $entry['name'] ) ? str_format( $entry['name'] ) : '' ) .
		'" />' . "\n";
	$HTML .= '</div>' . "\n";
	$HTML .= '<div class="required">' . "\n";
	$HTML .= '	<label for="contest-email">Email address</label>' . "\n";
	$HTML .= '	<input type="text" id="contest-email" name="contest[email]" value="' .
		( isset( $entry['email'] ) ? str_format( $entry['email'] ) : '' ) .
		'" />' . "\n";
	$HTML .= '</div>' . "\n";
	$HTML .= '</fieldset>' . "\n";

	$HTML .= '<fieldset>' . "\n";
	$HTML .= '<legend>' . ( $num_questions > 1 ? 'Questions' : 'Question' ) . '</legend>' . "\n";
	for ( $i = 0; $i < $num_questions; $i++ )
	{
		$HTML .= '<div class="required">' . "\n";
		$HTML .= '	<label for="contest-q' . $scoreboard['questions'][$i]['number'] . '">' .
			str_format( $scoreboard['questions'][$i]['name'] ) . '</label>' . "\n";
		$HTML .= '	<input type="text" name="contest[' . $scoreboard['questions'][$i]['number'] . ']" value="' .
			( isset( $entry[$i] ) ? str_format( $entry[$i] ) : '' ) .
			'" id="contest-q' . $scoreboard['questions'][$i]['number'] . '" />' . "\n";
		$HTML .= '</div>' . "\n";
	}
	$HTML .= '</fieldset>' . "\n";
	//$HTML .= '<span class="submit button">Submit!</span>' . "\n";
	$HTML .= '<button type="submit" value="Submit!">Submit!</button>' . "\n";
	$HTML .= '</form>' . "\n";

	return $HTML;
}


// contest is running
if ( contest_running() )
{
	// contest was submitted
	if ( $_POST['contest'] )
	{
		$entry = $_POST['contest'];
		// echo '<pre>', print_r($entry, true), '</pre>';
	
		// check that all fields were submitted
		$valid = true;
		foreach( $entry as $key=>$value )
		{
			$entry[$key] = trim( str_replace( "\t", ' ', $entry[$key] ) );
			if ( strlen( $entry[$key] ) == 0 )
				$valid = false;
		}
	
		if ( $valid )
		{
			// check the leaderboard and current submissions for usernames and emails
			$scored = find_user( $entry['name'], $entry['email'], 'scores_file' );
			$entered = find_user( $entry['name'], $entry['email'], 'entries_file' );
	
			if ( $scored == 'name' )
			{
				$Display = 'error';
				$Message = message( 'error', 'That name has already been registered; please choose a different name.' );
			}
			else if ( $scored == 'email' )
			{
				$Display = 'error';
				$Message = message( 'error', 'That email address has already been registered; perhaps you typed a different name to last time?' );
			}
			else if ( $entered == 'name' )
			{
				$Display = 'error';
				$Message = message( 'error', 'That name has been registered; please choose a different name.' );
			}
			else if ( $entered == 'email' || $entered == 'both' )
			{
				$Display = 'error';
				$Message = message( 'error', 'You have already entered this contest. Please wait until a new contest starts to enter again.' );
			}
			else
			{
				// okay to roll
				$len = count( $scoreboard['questions'] );
				for ( $i = 0; $i < $len; $i++ )
				{
					$submission[$i] = array( $entry['name'], $entry['email'], $i, $entry[$i], 'X' );
				}
				file_append( 'entries_file', $submission );
				$Display = 'success';
				$Message = message( 'message', 'Thank you for entering the contest.' );
			}
		}
		else
		{
			$Display = 'error';
			$Message = message( 'error', 'You did not fill in the form correctly. All fields are required.' );
		}
	}
	else
	{
		$Display = 'normal';
	}
}
// contest is not running
else
{
	$Display = 'success';
	$Message = message( 'message', 'There is no contest running at the moment. Please come back later.' );
}





if ( !$display_inline )
{
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">', "\n";
	echo '<html xmlns="http://www.w3.org/1999/xhtml" lang="en-gb" xml:lang="en-gb">', "\n\n";

	echo '<head>', "\n";
	echo '<title>Play the contest</title>', "\n";
	echo '<link rel="stylesheet" type="text/css" href="stylesheet.css" />', "\n";
	echo '<script type="text/javascript" src="js/jquery.js"></script>', "\n";
	echo '<script type="text/javascript" src="js/scoreboard.js"></script>', "\n";
	echo '<script type="text/javascript">$(document).ready(function(){ scoreboard_general() });</script>', "\n";
	echo '</head>', "\n\n";

	echo '<body>', "\n\n";
	
	echo '<h1>', $scoreboard['site_name'], '</h1>', "\n";
	echo '<div id="content-box">', "\n";
	echo '<h2>Play the contest</h2>', "\n";
}

if ( $Display == 'normal' )
{
	echo entry_form();
}
else if ( $Display == 'error' )
{
	echo $Message;
	echo entry_form( $entry );
}
else if ( $Display == 'success' )
{
	echo $Message;
}



if ( $display_inline )
{
	echo footer();
}
else
{
	echo '<p class="note"><a href="javascript:window.close()">CLOSE WINDOW</a></p>', "\n";
	echo '</div>', "\n";
	echo footer();
	echo '</body>', "\n";
	echo '</html>', "\n";
}

?>
