<?php
/*
 *	PHP Contest Scoreboard - run contests on your web site
 *
 *	Copyright (C) 2003-2007 Scott Vivian
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU General Public License for more details.
 */

/* Core variables
****************************/

// Password: You MUST change this to stop unauthorized people from messing with things!
$scoreboard['password'] = 'demo';

// Site name: displayed in pop up windows for contest entry and leaderboard.
// HTML can be used so you may put a code for a logo here if you wish.
$scoreboard['site_name'] = 'Agent Appreciation Week';

// Points: The number of points awarded per correct answer.
$scoreboard['contest_points'] = 1;

// Storage files
$cwf = str_replace( '\\', '/', __FILE__ ); // to fix Windows ;)
$cwd = substr( $cwf, 0, strrpos( $cwf, '/' ) );
$scoreboard['questions_file'] = $cwd . '/data/questions.php';
$scoreboard['entries_file'] = $cwd . '/data/entries.php';
$scoreboard['scores_file'] = $cwd . '/data/scores.php';

$scoreboard['cookie'] = 'scoreboard_login';

$scoreboard['file_protect'] = '<?php die(\'This file is protected. You are not allowed to view its contents.\'); ?>';

$scoreboard['version'] = '3.0';

// End of core variables


/* Global variables
****************************/

$scoreboard['questions'] = get_questions();


/*
	Checks if user is logged in
*/
function logged_in()
{
	global $scoreboard;
	return
		strlen( $_COOKIE[$scoreboard['cookie']] ) > 0 &&
		$_COOKIE[$scoreboard['cookie']] == md5($scoreboard['password']);
}


/*
	Outputs error message
*/
function message( $type, $message )
{
	return '<div class="info-box ' . $type . '"><p><strong>' . $message . '</strong></p></div>' . "\n";
}

/*
	Formats string to be displayed on web page
*/
function str_format( $s )
{
	return htmlspecialchars( stripslashes( $s ), ENT_QUOTES );
}


/*
	Checks if the contest is running (flag stoed by each question)
*/
function contest_running()
{
	global $scoreboard;
	return ( count( $scoreboard['questions'] ) > 0 && $scoreboard['questions'][0]['running'] == 'Y' );
}

function close_contest()
{
	global $scoreboard;

	$len = count( $scoreboard['questions'] );
	for ( $i = 0; $i < $len; $i++ )
	{
		$scoreboard['questions'][$i]['running'] = 'N';
	}
	file_write( 'questions_file', $scoreboard['questions'] );
}

/*
	Writes data from an array to a file, (truncating beforehand). Each
	entry in the array is stored as a line, attributes are tab-separated.
*/
function file_write( $file, $data=null )
{
	global $scoreboard;

	$fp = fopen( $scoreboard[$file], 'w' )
		or die('Could not open file for writing; make sure the file permissions are correct.');
	fwrite( $fp, $scoreboard['file_protect']."\n" );
	if ( $data != null )
	{
		foreach ( $data as $d )
		{
			$line = implode( "\t", $d ) . "\n";
			fwrite( $fp, $line );
		}
	}
	fclose( $fp );
}


/*
	Appends data from an array to a file. Each entry in the array is
	stored as a line, attributes are tab-separated.
*/
function file_append( $file, $data )
{
	global $scoreboard;

	$fp = fopen( $scoreboard[$file], 'a' );
	foreach ( $data as $d )
	{
		$line = implode( "\t", $d ) . "\n";
		fwrite( $fp, $line );
	}
	fclose( $fp );
}


/*
	Returns the specified question, or all questions if nothing specified
*/
function get_questions( $_question=0 )
{
	global $scoreboard;
	$fp = fopen( $scoreboard['questions_file'], 'r' );
	fgets( $fp );

	$questions = array();
	while ( !feof( $fp ) )
	{
		$q = null;
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			list( $q['number'], $q['name'], $q['running'] ) = explode( "\t", $buffer );
			$q = array_reverse( $q, true );

			if ( $_question == 0 || $_question == ( $q['number'] + 1 ) )
			{
				$questions[] = $q;
			}
		}
	}

	fclose( $fp );
	return $questions;
}


/*
	Returns entries for specified question
*/
function get_entries( $_question=0 )
{
	global $scoreboard;
	$fp = fopen( $scoreboard['entries_file'], 'r' );
	fgets( $fp );

	$entries = array();
	while ( !feof( $fp ) )
	{
		$e = null;
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			list( $e['name'], $e['email'], $e['question'], $e['answer'], $e['correct'] ) = explode( "\t", $buffer );
			$e = array_reverse( $e, true );

			if ( $_question == 0 || $_question == ( $e['question'] + 1 ) )
				$entries[] = $e;
		}
	}

	fclose( $fp );
	return $entries;
}


/*
	Updates entries in file
*/
function set_entries( $_entries )
{
	// primary key:	name/email/question
	// entries:		name/email/question/answer/correct
	global $scoreboard;
	$fp = fopen( $scoreboard['entries_file'], 'r' );
	fgets( $fp );

	$data = array();
	while ( !feof( $fp ) )
	{
		$e = null;
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			// key matched: update entry
			list( $e['name'], $e['email'], $e['question'], $e['answer'], $e['correct'] ) = explode( "\t", $buffer );
			$e = array_reverse( $e, true );

			if ( ($n=in_array_entries( $e, $_entries )) !== false )
			{
				$e = $_entries[$n];
			}
			$data[] = $e;
		}
	}
	fclose( $fp );

	file_write( 'entries_file', $data );
}


/*
	Gets entries from leaderboard
	name/email/score
*/
function get_scores()
{
	global $scoreboard;
	$fp = fopen( $scoreboard['scores_file'], 'r' );
	fgets( $fp );

	$scores = array();
	while ( !feof( $fp ) )
	{
		$s = null;
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			list( $s['name'], $s['email'], $s['points'] ) = explode( "\t", $buffer );
			$scores[] = array_reverse( $s, true );
		}
	}

	fclose( $fp );
	return $scores;
}


/*
	Writes all scores to the leaderboard
*/
function set_scores( $_entries )
{
	global $scoreboard;
	$scores = get_scores();

	$fp = fopen( $scoreboard['entries_file'], 'r' );
	fgets( $fp );

	while ( !feof( $fp ) )
	{
		$e = null;
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			list( $e['name'], $e['email'], $e['question'], $e['answer'], $e['correct'] ) = explode( "\t", $buffer );
			$e = array_reverse( $e, true );

			if ( $e['correct'] == 'Y' )
			{
				// loop through $scores: update user's score if they exist or create new user
				$user_found = false;
				$num_scores = count( $scores );
				for ( $i = 0; $i < $num_scores; $i++ )
				{
					if ( $scores[$i]['name'] == $e['name'] && $scores[$i]['email'] == $e['email'] )
					{
						$user_found = true;
						$scores[$i]['points'] += $scoreboard['contest_points'];
					}
				}
				// no user found: create one
				if ( !$user_found )
					$scores[$num_scores] = array( 'name'=>$e['name'], 'email'=>$e['email'], 'points'=>$scoreboard['contest_points'] );
			}
		}
	}
	fclose( $fp );

	file_write( 'scores_file', $scores );
}


/*
	Checks if entry is in array (KEY: name/email/question)
*/
function in_array_entries( $entry, $array )
{
	$len = count( $array );
	for ( $i = 0; $i < $len; $i++ )
	{
		if ( $entry['name'] == $array[$i]['name'] && $entry['email'] == $array[$i]['email'] && $entry['question'] == $array[$i]['question'] )
			return $i;
	}
	return false;
}


/*
	Comparison function for sorting user scores in descending order
*/
function cmp_scores( $a, $b )
{
	if ( $a['points'] == $b['points'] )
		return 0;
	return ( $a['points'] < $b['points'] ) ? 1 : -1;
}

/*
	Checks if the user exists in given file
*/
function find_user( $_name, $_email, $file )
{
	global $scoreboard;
	$fp = fopen( $scoreboard[$file], 'r' );
	fgets( $fp );

	while ( !feof( $fp ) )
	{
		$buffer = trim( fgets( $fp ) );
		if ( strlen( $buffer ) > 0 )
		{
			// get first two columns
			list( $name, $email ) = explode( "\t", $buffer );
			if ( $_name == $name && $_email == $email )
			{
				fclose( $fp );
				return 'both';
			}
			else if ( $_name == $name )
			{
				fclose( $fp );
				return 'name';
			}
			else if ( $_email == $email )
			{
				fclose( $fp );
				return 'email';
			}
		}
	}
	fclose( $fp );
	return 'neither';
}


/*
	Link to CodeLair may be removed as long as attribution is provided elsewhere.
*/
function footer( $admin=false )
{
	$HTML = '';
	if ( $admin && logged_in() )
		$HTML = '[ <a href="' . $_SERVER[PHP_SELF] . '?logout"><b>logout</b></a> ]<br /><br />' . $HTML;

	return '<p class="footer">' . $HTML . 'Powered by <a href="http://www.doheth.co.uk/codelair/">CodeLair PHP Contest Scoreboard</a></p>' . "\n";
}


?>
